@@ -20,6 +20,7 @@ CMAKE_BUILD_TYPE='RelWithDebInfo'
 DEBUG_COMMENT=''
 WITH_JEMALLOC=''
 COPYGALERA=0
+DEBUG_EXTNAME=''
 
 # Some programs that may be overriden
 TAR=${TAR:-tar}
@@ -47,6 +48,7 @@ do
         shift
         CMAKE_BUILD_TYPE='Debug'
         BUILD_COMMENT="${BUILD_COMMENT:-}-debug"
+        DEBUG_EXTNAME='-DDEBUG_EXTNAME=OFF'
         ;;
     -G | --copygalera)
         shift
@@ -141,7 +143,7 @@ fi
 
 WSREP_REV="$(cd "$SOURCEDIR";test -r WSREP-REVISION && cat WSREP-REVISION || echo "$REVISION")"
 if [[ $COPYGALERA -eq 0 ]];then 
-    GALERA_REVISION="$(cd "$SOURCEDIR/percona-xtradb-cluster-galera"; test -r GALERA-REVISION && cat GALERA-REVISION || bzr revno)"
+    GALERA_REVISION="$(cd "$SOURCEDIR/percona-xtradb-cluster-galera"; test -r GALERA-REVISION && cat GALERA-REVISION)"
 fi
 PRODUCT_FULL="$PRODUCT-$RELEASE_TAG$WSREP_VERSION.$REVISION.$(uname -s).$TARGET"
 COMMENT="Percona XtraDB Cluster (GPL) $MYSQL_VERSION-$RELEASE_TAG$WSREP_VERSION"
@@ -170,6 +172,14 @@ then
 
 fi
 
+if [[ -n ${LDFLAGS:-} ]];then 
+   MLDFLAGS=-DWITH_MYSQLD_LDFLAGS=$LDFLAGS 
+   WITHASAN=ON
+else 
+   MLDFLAGS=""
+   WITHASAN=OFF
+fi
+
 # Build
 (
     cd "$SOURCEDIR"
@@ -181,17 +191,17 @@ fi
         export CXX=${GALERA_CXX:-g++}
 
         cd "percona-xtradb-cluster-galera"
-        scons --config=force revno="$GALERA_REVISION" $MAKE_JFLAG \
+        scons --config=force boost_pool=0 revno="$GALERA_REVISION" $MAKE_JFLAG \
               garb/garbd libgalera_smm.so
         mkdir -p "$WORKDIR/usr/local/$PRODUCT_FULL/bin" \
              "$WORKDIR/usr/local/$PRODUCT_FULL/lib"
         cp garb/garbd "$WORKDIR/usr/local/$PRODUCT_FULL/bin"
         cp libgalera_smm.so "$WORKDIR/usr/local/$PRODUCT_FULL/lib"
     else 
-        mkdir -p $WORKDIR/usr/local/$PRODUCT_FULL/lib/{galera2,galera3}
-        #cp $WORKDIR/garbd "$WORKDIR/usr/local/$PRODUCT_FULL/bin"
-        cp $WORKDIR/galera2/libgalera_smm.so "$WORKDIR/usr/local/$PRODUCT_FULL/lib/galera2/"
-        cp $WORKDIR/galera3/libgalera_smm.so "$WORKDIR/usr/local/$PRODUCT_FULL/lib/galera3/"
+        mkdir -p "$WORKDIR/usr/local/$PRODUCT_FULL/bin" \
+             "$WORKDIR/usr/local/$PRODUCT_FULL/lib"
+        cp $WORKDIR/garbd "$WORKDIR/usr/local/$PRODUCT_FULL/bin"
+        cp $WORKDIR/libgalera_smm.so "$WORKDIR/usr/local/$PRODUCT_FULL/lib"
     fi
 
     ) || exit 1
@@ -199,11 +209,15 @@ fi
 
     #make -f Makefile-pxc all
 
-    cmake . ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
-        -DCMAKE_BUILD_TYPE="$CMAKE_BUILD_TYPE" \
+    cmake . ${CMAKE_OPTS:-} $MLDFLAGS -DBUILD_CONFIG=mysql_release \
+        -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE:-RelWithDebInfo} \
+        $DEBUG_EXTNAME \
+        -DWITH_ASAN=$WITHASAN \
         -DWITH_EMBEDDED_SERVER=OFF \
         -DFEATURE_SET=community \
+        -DENABLE_DTRACE=OFF \
         -DWITH_SSL=system \
+        -DMYSQL_MAINTAINER_MODE=OFF \
         -DWITH_ZLIB=system \
         -DCMAKE_INSTALL_PREFIX="/usr/local/$PRODUCT_FULL" \
         -DMYSQL_DATADIR="/usr/local/$PRODUCT_FULL/data" \
@@ -217,19 +231,6 @@ fi
     make $MAKE_JFLAG $QUIET -k
     make DESTDIR="$WORKDIR" install
 
-    # Build HandlerSocket
-    (
-        cd "plugin/HandlerSocket-Plugin-for-MySQL"
-        ./autogen.sh
-        CXX=${HS_CXX:-g++} ./configure --with-mysql-source="$SOURCEDIR/" \
-            --with-mysql-bindir="$SOURCEDIR/scripts" \
-            --with-mysql-plugindir="/usr/local/$PRODUCT_FULL/lib/mysql/plugin" \
-            --libdir="/usr/local/$PRODUCT_FULL/lib/mysql/plugin" \
-            --prefix="/usr/local/$PRODUCT_FULL"
-        make $MAKE_JFLAG
-        make DESTDIR="$WORKDIR" install
-
-    )
 
     (
        echo "Packaging the test files"
@@ -243,11 +244,11 @@ fi
     (
         cd "$JEMALLOCDIR"
 
-        CFLAGS= ./autogen.sh --disable-valgrind --prefix="/usr/local/$PRODUCT_FULL/" \
+        ./configure --prefix="/usr/local/$PRODUCT_FULL/" \
                 --libdir="/usr/local/$PRODUCT_FULL/lib/mysql/"
         make $MAKE_JFLAG
         make DESTDIR="$WORKDIR" install_lib_shared
-        strip lib/libjemalloc* || true
+
         # Copy COPYING file
         cp COPYING "$WORKDIR/usr/local/$PRODUCT_FULL/COPYING-jemalloc"
 
