@@ -291,6 +291,7 @@ fi
         -DMYSQL_SERVER_SUFFIX="-$RELEASE_TAG$WSREP_VERSION" \
         -DWITH_INNODB_DISALLOW_WRITES=ON \
         -DWITH_WSREP=ON \
+        -DMYSQL_MAINTAINER_MODE=OFF \
         -DCOMPILATION_COMMENT="$COMMENT" \
         -DWITH_PAM=ON \
         -DWITH_INNODB_MEMCACHED=ON \
@@ -303,24 +304,25 @@ fi
         make clean
         cmake . ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
             -DCMAKE_BUILD_TYPE=Debug \
-            $DEBUG_EXTNAME \
+            -DDEBUG_EXTNAME=ON \
             -DWITH_EMBEDDED_SERVER=OFF \
             -DFEATURE_SET=community \
             -DENABLE_DTRACE=OFF \
+            -DMYSQL_MAINTAINER_MODE=OFF \
             $SSL_OPT \
             -DCMAKE_INSTALL_PREFIX="/usr/local/$PRODUCT_FULL" \
             -DMYSQL_DATADIR="/usr/local/$PRODUCT_FULL/data" \
             -DMYSQL_SERVER_SUFFIX="-$RELEASE_TAG$WSREP_VERSION" \
             -DWITH_INNODB_DISALLOW_WRITES=ON \
             -DWITH_WSREP=ON \
-            -DCOMPILATION_COMMENT="$COMMENT" \
+            -DCOMPILATION_COMMENT="$COMMENT - UNIV_DEBUG ON" \
             -DWITH_PAM=ON \
             -DWITH_INNODB_MEMCACHED=ON \
             $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
         make $MAKE_JFLAG $QUIET
 
         echo "Copying mysqld-debug"
-        cp -v sql/mysqld-debug $WORKDIR/usr/local/$PRODUCT_FULL/bin/
+        cp -v sql/mysqld-debug $WORKDIR/usr/local/$PRODUCT_FULL/bin/mysqld-debug
     fi
 
 
@@ -353,14 +355,14 @@ fi
 (
     cd "$WORKDIR/usr/local/"
 
-    find $PRODUCT_FULL ! -type d  ! \( -iname '*toku*' -o -iwholename '*/tokudb*/*' -o -iname '*tdb*.h' \) | sort > $WORKDIR/tokudb_server.list
+    find $PRODUCT_FULL ! -type d  ! \( -iname '*toku*' -o -iwholename '*/tokudb*/*'  \) | sort > $WORKDIR/tokudb_server.list
     $TAR --owner=0 --group=0 -czf "$WORKDIR/$PRODUCT_FULL.tar.gz" -T $WORKDIR/tokudb_server.list
     rm -f $WORKDIR/tokudb_server.list
 
     if test -e "$PRODUCT_FULL/lib/mysql/plugin/ha_tokudb.so"
     then
         TARGETTOKU=$(echo $PRODUCT_FULL | sed 's/.Linux/.TokuDB.Linux/')
-	find $PRODUCT_FULL ! -type d \( -iname '*toku*' -o -iwholename '*/tokudb*/*' -o -iname '*tdb*.h' \) > $WORKDIR/tokudb_plugin.list
+	find $PRODUCT_FULL ! -type d \( -iname '*toku*' -o -iwholename '*/tokudb*/*' \) > $WORKDIR/tokudb_plugin.list
         $TAR --owner=0 --group=0 -czf "$WORKDIR/$TARGETTOKU.tar.gz" -T $WORKDIR/tokudb_plugin.list
         rm -f $WORKDIR/tokudb_plugin.list
     fi
