@@ -17,8 +17,12 @@ MYSQLD=
 niceness=0
 mysqld_ld_preload=
 mysqld_ld_library_path=
+load_jemalloc=1
 flush_caches=0
 numa_interleave=0
+resume_on_fail=1
+# Change (disable) transparent huge pages (TokuDB requirement)
+thp_setting=
 
 # Initial logging status: error log is not open, and not using syslog
 logging=init
@@ -67,6 +71,14 @@ case "$1" in
 esac
 
 usage () {
+        thp_usage=""
+        if [ -f /sys/kernel/mm/transparent_hugepage/enabled ]; then
+          thp_usage=$(cat <<'EOF'
+  --thp-setting=SETTING      Change transparent huge pages setting
+                             on the system before starting mysqld
+EOF
+);
+        fi
         cat <<EOF
 Usage: $0 [OPTIONS]
   --no-defaults              Don't read the system defaults file
@@ -90,6 +102,7 @@ Usage: $0 [OPTIONS]
                              starting the server
   --numa-interleave          Run mysqld with its memory interleaved
                              on all NUMA nodes
+${thp_usage}
 
 All other options are passed to the mysqld program.
 
@@ -251,8 +264,8 @@ wsrep_recover_position() {
   if [ -z "$rp" ]; then
     local skipped="$(grep WSREP $wr_logfile | grep 'skipping position recovery')"
     if [ -z "$skipped" ]; then
-      log_error "WSREP: Failed to recover position: " `cat $wr_logfile`;
-      ret=1
+      log_error "WSREP: Failed to recover position: "
+      ret=2
     else
       log_notice "WSREP: Position recovery skipped"
     fi
@@ -263,6 +276,15 @@ wsrep_recover_position() {
     wsrep_start_position_opt="--wsrep_start_position=$start_pos"
   fi
 
+  if test -f $err_log && test $logging = 'file';then 
+      echo "Log of wsrep recovery (--wsrep-recover):" >> $err_log
+      cat $wr_logfile >> $err_log
+  elif test $logging = 'syslog';then
+      logger -t "$syslog_tag_mysqld_safe" -p "$priority" \
+          "Log of wsrep recovery (--wsrep-recover):"
+      logger -t "$syslog_tag_mysqld_safe" -p "$priority" < $wr_logfile
+  fi
+
   rm $wr_logfile
 
   return $ret
@@ -304,7 +326,10 @@ parse_arguments() {
       # mysqld_safe-specific options - must be set in my.cnf ([mysqld_safe])!
       --core-file-size=*) core_file_size="$val" ;;
       --ledir=*) ledir="$val" ;;
-      --malloc-lib=*) set_malloc_lib "$val" ;;
+      --malloc-lib=*)
+	set_malloc_lib "$val"
+	load_jemalloc=0
+	;;
       --mysqld=*) MYSQLD="$val" ;;
       --mysqld-version=*)
         if test -n "$val"
@@ -319,6 +344,7 @@ parse_arguments() {
       --open-files-limit=*) open_files="$val" ;;
       --open_files_limit=*) open_files="$val" ;;
       --skip-kill-mysqld*) KILL_MYSQLD=0 ;;
+      --thp-setting=*) thp_setting="$val" ;;
       --syslog) want_syslog=1 ;;
       --skip-syslog) want_syslog=0 ;;
       --syslog-tag=*) syslog_tag="$val" ;;
@@ -327,6 +353,7 @@ parse_arguments() {
       --wsrep-data-home-dir=*) wsrep_data_home_dir="$val"; ;;
       --flush-caches=*) flush_caches="$val" ;;
       --numa-interleave=*) numa_interleave="$val" ;;
+      --exit-on-recover-fail) resume_on_fail=0 ;;
       --wsrep[-_]provider=*)
         if test -n "$val" && test "$val" != "none"
         then
@@ -472,19 +499,6 @@ set_malloc_lib() {
 }
 
 
-#
-# Add jemalloc to ld_preload - needed for TokuDB
-#
-for libjemall in "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"
-do
-if test -f "$libjemall/libjemalloc.so.1"
-then
-  add_mysqld_ld_preload "$libjemall/libjemalloc.so.1"
-  break
-fi
-done
-
-
 #
 # First, try to find BASEDIR and ledir (where mysqld is)
 #
@@ -614,6 +628,19 @@ fi
 parse_arguments `$print_defaults $defaults --loose-verbose mysqld_safe safe_mysqld`
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
+#
+# Add jemalloc to ld_preload if no other malloc forced - needed for TokuDB
+#
+if test $load_jemalloc -eq 1
+then
+  for libjemall in "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+    if [ -r "$libjemall/libjemalloc.so.1" ]; then
+      add_mysqld_ld_preload "$libjemall/libjemalloc.so.1"
+      break
+    fi  
+  done
+fi
+
 #
 # Try to find the plugin directory
 #
@@ -893,6 +920,53 @@ then
   exit 1
 fi
 
+# If thp-setting is specified, check to see if thp is supported
+# on this kernel and clear the value if it isn't
+if [ -n "$thp_setting" ] && [ ! -f /sys/kernel/mm/transparent_hugepage/enabled ]
+then
+  log_notice "Transparent huge pages is not supported on this system, ignoring thp-setting."
+  thp_setting=
+fi
+
+# Change transparent huge pages setting if thp-setting option specified
+if [ -n "$thp_setting" ]
+then
+  if [ $(id -u) -ne 0 ]; then
+    log_error "mysqld_safe must be run as root for setting transparent huge pages!"
+    exit 1
+  elif [ $thp_setting != "always" -a $thp_setting != "madvise" -a $thp_setting != "never" ]; then
+    log_error "Invalid value for thp-setting=$thp_setting in config file. Valid values are: always, madvise or never"
+    exit 1
+  else
+    if [ -f /sys/kernel/mm/transparent_hugepage/enabled ]; then
+      CONTENT_THP=$(cat /sys/kernel/mm/transparent_hugepage/enabled)
+      STATUS_THP=0
+      set +e
+      STATUS_THP=$(echo $CONTENT_THP | grep -cv "\[${thp_setting}\]")
+      set -e
+    fi
+    if [ $STATUS_THP -eq 0 ]; then
+      log_notice "Transparent huge pages are already set to: ${thp_setting}."
+    else
+      if [ -f /sys/kernel/mm/transparent_hugepage/defrag ]; then
+        echo $thp_setting > /sys/kernel/mm/transparent_hugepage/defrag
+        if [ $? -ne 0 ]; then
+	  log_error "Error setting transparent huge pages to: ${thp_setting}."
+          exit 1
+        fi
+      fi
+      if [ -f /sys/kernel/mm/transparent_hugepage/enabled ]; then
+        echo $thp_setting > /sys/kernel/mm/transparent_hugepage/enabled
+        if [ $? -ne 0 ]; then
+	  log_error "Error setting transparent huge pages to: ${thp_setting}."
+          exit 1
+        fi
+      fi
+      log_notice "Successfuly set transparent huge pages to: ${thp_setting}."
+    fi
+  fi
+fi
+
 #
 # Uncomment the following lines if you want all tables to be automatically
 # checked and repaired during startup. You should add sensible key_buffer
@@ -988,8 +1062,20 @@ do
     wsrep_start_position_opt="--wsrep_start_position='00000000-0000-0000-0000-000000000000:-1'"
   fi
 
+  retcode=$?
 
-  [ $? -ne 0 ] && exit 1 #
+  if test $retcode -eq 2;then
+      if  test $resume_on_fail -eq 1;then
+        log_notice "wsrep-recovery has failed to recover, continuing with startup"
+        wsrep_start_position_opt=""
+      else 
+        log_error " --exit-on-recover-fail is provided, bailing out"
+        exit 2
+      fi
+  elif test $retcode -ne 0;then
+      log_error "Unknown error: $retcode"
+      exit $retcode
+  fi
 
   [ -n "$wsrep_urls" ] && url=`wsrep_pick_url $wsrep_urls` # check connect address
 
