@@ -37,6 +37,8 @@
 #include "sql_acl.h"            // SUPER_ACL
 #include "sql_audit.h"
 #include "mysql/service_my_plugin_log.h"
+#include "sp_rcontext.h"
+#include "sp_head.h"
 
 #include <my_dir.h>
 #include <stdarg.h>
@@ -1568,6 +1570,7 @@ bool MYSQL_LOG::open(
                      bool unique)
 {
   char buff[FN_REFLEN];
+  MY_STAT f_stat;
   File file= -1;
   my_off_t pos= 0;
   int open_flags= O_CREAT | O_BINARY;
@@ -1587,6 +1590,10 @@ bool MYSQL_LOG::open(
       DBUG_EVALUATE_IF("fault_injection_init_name", log_type == LOG_BIN, 0))
     goto err;
 
+  /* File is regular writable file */
+  if (my_stat(log_file_name, &f_stat, MYF(0)) && !MY_S_ISREG(f_stat.st_mode))
+    goto err;
+
   if (io_cache_type == SEQ_READ_APPEND)
     open_flags |= O_RDWR | O_APPEND;
   else
@@ -1645,7 +1652,7 @@ bool MYSQL_LOG::open(
   DBUG_RETURN(0);
 
 err:
-  if (log_type == LOG_BIN && binlogging_impossible_mode == ABORT_SERVER)
+  if (log_type == LOG_BIN && binlog_error_action == ABORT_SERVER)
   {
     THD *thd= current_thd;
     /*
@@ -2082,6 +2089,13 @@ bool MYSQL_QUERY_LOG::write(THD *thd, ulonglong current_utime,
     if (my_b_write(&log_file, (uchar*) "\n", 1))
         tmp_errno= errno;
 
+    if (opt_log_slow_sp_statements &&
+        thd->sp_runtime_ctx &&
+        my_b_printf(&log_file,
+                    "# Stored_routine: %s\n",
+                    thd->sp_runtime_ctx->sp->m_qname.str) == (uint) -1)
+      tmp_errno= errno;
+
 #if defined(ENABLED_PROFILING)
     thd->profiling.print_current(&log_file);
 #endif
