@@ -115,7 +115,7 @@ extern bool opt_disable_networking, opt_skip_show_db;
 extern bool opt_skip_name_resolve;
 extern bool opt_ignore_builtin_innodb;
 extern my_bool opt_character_set_client_handshake;
-extern bool volatile abort_loop;
+extern MYSQL_PLUGIN_IMPORT bool volatile abort_loop;
 extern bool in_bootstrap;
 extern my_bool opt_bootstrap;
 extern uint connection_count;
@@ -124,7 +124,7 @@ extern my_bool opt_safe_show_db, opt_local_infile, opt_myisam_use_mmap;
 extern my_bool opt_slave_compressed_protocol, use_temp_pool;
 extern ulong slave_exec_mode_options;
 extern ulonglong slave_type_conversions_options;
-extern my_bool read_only, opt_readonly;
+extern my_bool read_only, opt_readonly, super_read_only, opt_super_readonly;
 extern my_bool lower_case_file_system;
 extern ulonglong slave_rows_search_algorithms_options;
 #ifndef DBUG_OFF
@@ -236,15 +236,16 @@ extern const char *binlog_checksum_type_names[];
 extern my_bool opt_master_verify_checksum;
 extern my_bool opt_slave_sql_verify_checksum;
 extern my_bool enforce_gtid_consistency;
-extern ulong binlogging_impossible_mode;
-enum enum_binlogging_impossible_mode
+extern my_bool simplified_binlog_gtid_recovery;
+extern ulong binlog_error_action;
+enum enum_binlog_error_action
 {
   /// Ignore the error and let server continue without binlogging
   IGNORE_ERROR= 0,
   /// Abort the server
   ABORT_SERVER= 1
 };
-extern const char *binlogging_impossible_err[];
+extern const char *binlog_error_action_list[];
 enum enum_gtid_mode
 {
   /// Support only anonymous groups, not GTIDs.
@@ -257,6 +258,8 @@ enum enum_gtid_mode
   GTID_MODE_ON= 3
 };
 extern ulong gtid_mode;
+extern bool gtid_deployment_step;
+extern bool opt_gtid_deployment_step;
 extern const char *gtid_mode_names[];
 extern TYPELIB gtid_mode_typelib;
 
@@ -662,6 +665,7 @@ enum options_mysqld
   OPT_BINLOG_FORMAT,
   OPT_BINLOG_IGNORE_DB,
   OPT_BIN_LOG,
+  OPT_BINLOGGING_IMPOSSIBLE_MODE,
   OPT_BOOTSTRAP,
   OPT_CONSOLE,
   OPT_DEBUG_SYNC_TIMEOUT,
@@ -697,6 +701,7 @@ enum options_mysqld
   OPT_SSL_CERT,
   OPT_SSL_CIPHER,
   OPT_SSL_KEY,
+  OPT_THREAD_CONCURRENCY,
   OPT_UPDATE_LOG,
   OPT_WANT_CORE,
   OPT_ENGINE_CONDITION_PUSHDOWN,
