@@ -1233,6 +1233,12 @@ create_view_query(THD *thd, uchar** buf, size_t* buf_len)
     return wsrep_to_buf_helper(thd, buff.ptr(), buff.length(), buf, buf_len);
 }
 
+/*
+  returns: 
+   0: statement was replicated as TOI
+   1: TOI replication was skipped
+  -1: TOI replication failed 
+ */
 static int wsrep_TOI_begin(THD *thd, char *db_, char *table_,
                            const TABLE_LIST* table_list)
 {
@@ -1271,31 +1277,39 @@ static int wsrep_TOI_begin(THD *thd, char *db_, char *table_,
   struct wsrep_buf buff = { buf, buf_len };
   if (WSREP(thd))
       thd_proc_info(thd, "Preparing for TO isolation");
-  if (!buf_err                                                    &&
+  if (!buf_err                                                                &&
       wsrep_prepare_keys_for_isolation(thd, db_, table_, table_list, &key_arr)&&
+      key_arr.keys_len > 0                                                    &&
       WSREP_OK == (ret = wsrep->to_execute_start(wsrep, thd->thread_id,
-                                                 key_arr.keys, key_arr.keys_len,
-                                                 &buff, 1,
-                                                 &thd->wsrep_trx_meta)))
+						 key_arr.keys, key_arr.keys_len,
+						 &buff, 1,
+						 &thd->wsrep_trx_meta)))
   {
     thd->wsrep_exec_mode= TOTAL_ORDER;
     wsrep_to_isolation++;
     if (buf) my_free(buf);
     wsrep_keys_free(&key_arr);
     WSREP_DEBUG("TO BEGIN: %lld, %d",(long long)wsrep_thd_trx_seqno(thd),
-                thd->wsrep_exec_mode);
+		thd->wsrep_exec_mode);
   }
-  else {
+  else if (key_arr.keys_len > 0) {
     /* jump to error handler in mysql_execute_command() */
     WSREP_WARN("TO isolation failed for: %d, sql: %s. Check wsrep "
                "connection state and retry the query.",
                ret, (thd->query()) ? thd->query() : "void");
     my_error(ER_LOCK_DEADLOCK, MYF(0), "WSREP replication failed. Check "
-            "your wsrep connection state and retry the query.");
+	     "your wsrep connection state and retry the query.");
     if (buf) my_free(buf);
     wsrep_keys_free(&key_arr);
     return -1;
   }
+  else {
+    /* non replicated DDL, affecting temporary tables only */
+    WSREP_DEBUG("TO isolation skipped for: %d, sql: %s."
+		"Only temporary tables affected.",
+		ret, (thd->query()) ? thd->query() : "void");
+    return 1;
+  }
   return 0;
 }
 
@@ -1395,13 +1409,45 @@ static void wsrep_RSU_end(THD *thd)
 int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
                              const TABLE_LIST* table_list)
 {
-
+  int ret= 0;
+  LEX *lex;
   /*
     No isolation for applier or replaying threads.
    */
   if (thd->wsrep_exec_mode == REPL_RECV) return 0;
 
-  int ret= 0;
+
+  lex= thd->lex;
+  if (!wsrep_replicate_myisam)
+  {
+    if (table_ && lex->create_info.db_type && \
+        lex->create_info.db_type->db_type == DB_TYPE_MYISAM)
+    {
+        DBUG_ASSERT(db_);
+        WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", db_, table_);
+        return ret;
+    } 
+    else if (table_list)
+    {
+        for (const TABLE_LIST *table= table_list; table; table= table->next_global)
+        {
+            /*
+             * First condition is required for IF EXISTS queries
+             */
+            if (table->table && table->table->file->ht->db_type == DB_TYPE_MYISAM)
+            {
+                WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", table->db, table->table_name);
+                /* 
+                 * Break even if one table is MyISAM
+                 * Note: This may not work well with CREATE TABLE .. SELECT (CTAS)
+                 * but CTAS is not executed as TOI currently.
+                 */
+                return ret;
+            }
+        }
+    }
+  }
+
   mysql_mutex_lock(&thd->LOCK_wsrep_thd);
 
   if (thd->wsrep_conflict_state == MUST_ABORT)
@@ -1449,9 +1495,15 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
                                                table_list); break;
     case WSREP_OSU_RSU: ret =  wsrep_RSU_begin(thd, db_, table_); break;
     }
-    if (!ret)
-    {
-      thd->wsrep_exec_mode= TOTAL_ORDER;
+    switch (ret) {
+    case 0:  thd->wsrep_exec_mode= TOTAL_ORDER; break;
+    case 1: 
+      /* TOI replication skipped, treat as success */ 
+      ret = 0; 
+      break;
+    case -1:
+      /* TOI replication failed, treat as error */ 
+      break;
     }
   }
   return ret;
