@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "wsrep_sst.h"
 
@@ -34,6 +34,7 @@ extern const char wsrep_defaults_file[];
 #define WSREP_SST_OPT_DATA     "--datadir"
 #define WSREP_SST_OPT_CONF     "--defaults-file"
 #define WSREP_SST_OPT_PARENT   "--parent"
+#define WSREP_SST_OPT_BINLOG   "--binlog"
 
 // mysqldump-specific options
 #define WSREP_SST_OPT_USER     "--user"
@@ -47,9 +48,9 @@ extern const char wsrep_defaults_file[];
 #define WSREP_SST_OPT_GTID     "--gtid"
 #define WSREP_SST_OPT_BYPASS   "--bypass"
 
-#define WSREP_SST_MYSQLDUMP    "mysqldump"
-#define WSREP_SST_RSYNC        "rsync"
-#define WSREP_SST_SKIP         "skip"
+#define WSREP_SST_MYSQLDUMP       "mysqldump"
+#define WSREP_SST_RSYNC           "rsync"
+#define WSREP_SST_SKIP            "skip"
 #define WSREP_SST_XTRABACKUP      "xtrabackup"
 #define WSREP_SST_XTRABACKUP_V2   "xtrabackup-v2"
 #define WSREP_SST_DEFAULT      WSREP_SST_XTRABACKUP_V2
@@ -109,7 +110,7 @@ bool  wsrep_sst_receive_address_check (sys_var *self, THD* thd, set_var* var)
     return 0;
 }
 
-bool wsrep_sst_receive_address_update (sys_var *self, THD* thd, 
+bool wsrep_sst_receive_address_update (sys_var *self, THD* thd,
                                        enum_var_type type)
 {
     return 0;
@@ -250,6 +251,7 @@ void wsrep_sst_received (wsrep_t*            const wsrep,
     wsrep_gtid_t const state_id = {
         *uuid, (rcode ? WSREP_SEQNO_UNDEFINED : seqno)
     };
+    wsrep_init_sidno(state_id.uuid);
     wsrep->sst_received(wsrep, &state_id, state, state_len, rcode);
 }
 
@@ -316,6 +318,33 @@ static char* my_fgets (char* buf, size_t buf_len, FILE* stream)
    return ret;
 }
 
+/*
+  Generate opt_binlog_opt_val for sst_donate_other(), sst_prepare_other().
+
+  Returns zero on success, negative error code otherwise.
+
+  String containing binlog name is stored in param ret if binlog is enabled
+  and GTID mode is on, otherwise empty string. Returned string should be
+  freed with my_free().
+ */
+static int generate_binlog_opt_val(char** ret)
+{
+  DBUG_ASSERT(ret);
+  *ret= NULL;
+  if (opt_bin_log && gtid_mode > 0)
+  {
+    assert(opt_bin_logname);
+    *ret= strcmp(opt_bin_logname, "0") ?
+        my_strdup(opt_bin_logname, MYF(0)) : my_strdup("", MYF(0));
+  }
+  else
+  {
+    *ret= my_strdup("", MYF(0));
+  }
+  if (!*ret) return -ENOMEM;
+  return 0;
+}
+
 static void* sst_joiner_thread (void* a)
 {
   sst_thread_arg* arg= (sst_thread_arg*) a;
@@ -410,17 +439,32 @@ static ssize_t sst_prepare_other (const char*  method,
   char    cmd_str[1024];
   const ssize_t cmd_len= sizeof(cmd_str);
   const char* sst_dir= mysql_real_data_home;
+  const char* binlog_opt= "";
+  char* binlog_opt_val= NULL;
 
-  int ret= snprintf (cmd_str, cmd_len,
-                     "wsrep_sst_%s "
-                     WSREP_SST_OPT_ROLE" 'joiner' "
-                     WSREP_SST_OPT_ADDR" '%s' "
-                     WSREP_SST_OPT_AUTH" '%s' "
-                     WSREP_SST_OPT_DATA" '%s' "
-                     WSREP_SST_OPT_CONF" '%s' "
-                     WSREP_SST_OPT_PARENT" '%d'",
-                     method, addr_in, (sst_auth_real) ? sst_auth_real : "",
-                     sst_dir, wsrep_defaults_file, (int)getpid());
+  int ret;
+  if ((ret= generate_binlog_opt_val(&binlog_opt_val)))
+  {
+    WSREP_ERROR("sst_prepare_other(): generate_binlog_opt_val() failed: %d",
+                ret);
+    return ret;
+  }
+  if (strlen(binlog_opt_val)) binlog_opt= WSREP_SST_OPT_BINLOG;
+
+
+  ret= snprintf (cmd_str, cmd_len,
+                 "wsrep_sst_%s "
+                 WSREP_SST_OPT_ROLE" 'joiner' "
+                 WSREP_SST_OPT_ADDR" '%s' "
+                 WSREP_SST_OPT_AUTH" '%s' "
+                 WSREP_SST_OPT_DATA" '%s' "
+                 WSREP_SST_OPT_CONF" '%s' "
+                 WSREP_SST_OPT_PARENT" '%d'"
+                 " %s '%s' ",
+                 method, addr_in, (sst_auth_real) ? sst_auth_real : "",
+                 sst_dir, wsrep_defaults_file, (int)getpid(),
+                 binlog_opt, binlog_opt_val);
+  my_free(binlog_opt_val);
 
   if (ret < 0 || ret >= cmd_len)
   {
@@ -455,7 +499,6 @@ static ssize_t sst_prepare_other (const char*  method,
   return ret;
 }
 
-//extern ulong my_bind_addr;
 extern uint  mysqld_port;
 
 /*! Just tells donor where to send mysqldump */
@@ -715,12 +758,11 @@ static int sst_donate_mysqldump (const char*         addr,
               WSREP_SST_OPT_PORT" '%s' "
               WSREP_SST_OPT_LPORT" '%u' "
               WSREP_SST_OPT_SOCKET" '%s' "
-              WSREP_SST_OPT_DATA" '%s' "
               WSREP_SST_OPT_CONF" '%s' "
               WSREP_SST_OPT_GTID" '%s:%lld'"
               "%s",
               user, pswd, host, port, mysqld_port, mysqld_unix_port,
-              mysql_real_data_home, wsrep_defaults_file, uuid_str,
+              wsrep_defaults_file, uuid_str,
               (long long)seqno, bypass ? " "WSREP_SST_OPT_BYPASS : "");
 
     WSREP_DEBUG("Running: '%s'", cmd_str);
@@ -751,10 +793,10 @@ static int run_sql_command(THD *thd, const char *query)
   mysql_parse(thd, thd->query(), thd->query_length(), &ps);
   if (thd->is_error())
   {
-    int const err= thd->stmt_da->sql_errno();
+    int const err= thd->get_stmt_da()->sql_errno();
     WSREP_WARN ("error executing '%s': %d (%s)%s",
-                query, err, thd->stmt_da->message(),
-                err == ER_UNKNOWN_SYSTEM_VARIABLE ? 
+                query, err, thd->get_stmt_da()->message(),
+                err == ER_UNKNOWN_SYSTEM_VARIABLE ?
                 ". Was mysqld built with --with-innodb-disallow-writes ?" : "");
     thd->clear_error();
     return -1;
@@ -776,8 +818,8 @@ static int sst_flush_tables(THD* thd)
   else
   {
     /* make sure logs are flushed after global read lock acquired */
-    err= reload_acl_and_cache(thd, REFRESH_ENGINE_LOG,
-                              (TABLE_LIST*) 0, &not_used);
+    err= reload_acl_and_cache(thd, REFRESH_ENGINE_LOG | REFRESH_BINARY_LOG,
+			      (TABLE_LIST*) 0, &not_used);
   }
 
   if (err)
@@ -930,6 +972,8 @@ wait_signal:
   return NULL;
 }
 
+
+
 static int sst_donate_other (const char*   method,
                              const char*   addr,
                              const char*   uuid,
@@ -938,21 +982,34 @@ static int sst_donate_other (const char*   method,
 {
   char    cmd_str[4096];
   const ssize_t cmd_len= sizeof(cmd_str);
+  const char* binlog_opt= "";
+  char* binlog_opt_val= NULL;
 
-  int ret= snprintf (cmd_str, cmd_len,
-                     "wsrep_sst_%s "
-                     WSREP_SST_OPT_ROLE" 'donor' "
-                     WSREP_SST_OPT_ADDR" '%s' "
-                     WSREP_SST_OPT_AUTH" '%s' "
-                     WSREP_SST_OPT_SOCKET" '%s' "
-                     WSREP_SST_OPT_DATA" '%s' "
-                     WSREP_SST_OPT_CONF" '%s' "
-                     WSREP_SST_OPT_GTID" '%s:%lld'"
-                     "%s",
-                     method, addr, sst_auth_real, mysqld_unix_port,
-                     mysql_real_data_home, wsrep_defaults_file,
-                     uuid, (long long) seqno,
-                     bypass ? " "WSREP_SST_OPT_BYPASS : "");
+  int ret;
+  if ((ret= generate_binlog_opt_val(&binlog_opt_val)))
+  {
+    WSREP_ERROR("sst_donate_other(): generate_binlog_opt_val() failed: %d",ret);
+    return ret;
+  }
+  if (strlen(binlog_opt_val)) binlog_opt= WSREP_SST_OPT_BINLOG;
+
+  ret= snprintf (cmd_str, cmd_len,
+                 "wsrep_sst_%s "
+                 WSREP_SST_OPT_ROLE" 'donor' "
+                 WSREP_SST_OPT_ADDR" '%s' "
+                 WSREP_SST_OPT_AUTH" '%s' "
+                 WSREP_SST_OPT_SOCKET" '%s' "
+                 WSREP_SST_OPT_DATA" '%s' "
+                 WSREP_SST_OPT_CONF" '%s' "
+                 " %s '%s' "
+                 WSREP_SST_OPT_GTID" '%s:%lld'"
+                 "%s",
+                 method, addr, sst_auth_real, mysqld_unix_port,
+                 mysql_real_data_home, wsrep_defaults_file,
+                 binlog_opt, binlog_opt_val,
+                 uuid, (long long) seqno,
+                 bypass ? " "WSREP_SST_OPT_BYPASS : "");
+  my_free(binlog_opt_val);
 
   if (ret < 0 || ret >= cmd_len)
   {
@@ -970,7 +1027,7 @@ static int sst_donate_other (const char*   method,
   {
     WSREP_ERROR("sst_donate_other(): pthread_create() failed: %d (%s)",
                 ret, strerror(ret));
-    return -ret;
+    return ret;
   }
   mysql_cond_wait (&arg.cond, &arg.lock);
 
